//============================================================================
// FILE: CoreConstantDescriptor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des divers descripteurs constants de Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CORECONSTDESCRIPTOR_HH
#define _CORECONSTDESCRIPTOR_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "ConstantDescriptor.h"

/**@name CDPFORTEXLes descripteurs prédéfinis
\begin{itemize} \item Directive d'inclusion :
[P]
\#include CoreConstantDescriptor.h
\#include CoreVariableDescriptor.h
[/P]
\end{itemize}

Quelques descripteurs, constants (C) et variables (V) sont prédéfinis dans DIESE. \\

Les deux premiers dans le tableau ci-dessous sont affectés par DIESE comme descripteur de la classe [C]DescriptedEntity[/C]. Les sous-classes de [C]DescriptedEntity[/C] dans les couches applicatives héritent donc de ces descripteurs.\\

Les autres sont des classes instantiables dans les couches logicielles applicatives :
\begin{verbatim}
[P]MyEntity::MyEntity() {
  SetMetaClass(MY_ENTITY);
  AddInstanceToClass();
  ...
}
AgeAttribute::AgeAttribute() {  // déclaré sous-classe de IntConstantDescriptor
  ClassName   ("ageAttribute");
  ClassSymbol (AGE_ATTRIBUTE);
  ...
};
int main(...) {
  ...
  Entity* pE = new MyEntity();
  Descriptor pD = new AgeAttribute();  // classe de l'application (voir ci-dessus)
  AddConstantDescriptor(pD);
  pD->SetToDefaultValue(-9);
  pD = new TypeAttribute();            // classe prédéfinie
  AddConstantDescriptor(pD);
  //pD = new InstantAttribute();       // automatiquement hérité
  ...
}[/P]
\end{verbatim}
 
\begin{tabular}{ccccll}
 & {\tiny type} & {\tiny classe} & {\tiny symbole de classe} & {\tiny v. par défaut}& {\tiny valeur}\\ \hline
\hline
{\tiny V} & {\tiny int} & {\tiny CurrentElementIndex} & {\tiny CURRENT_ELEMENT_INDEX} & {\tiny 0} & {\tiny 0}\\ \hline
{\tiny C} & {\tiny int} & {\tiny InstantAttribute} & {\tiny INSTANT_ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny int} & {\tiny TypeAttribute} & {\tiny TYPE_ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny int} & {\tiny DescriptorIdAttribute} & {\tiny DESCRIPTOR_ID_ATTRIBUTE} & {\tiny -9} & {\tiny -9}\\ \hline
{\tiny C} & {\tiny char*} & {\tiny DescriptorNameAttribute} & {\tiny DESCRIPTOR_NAME_ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny int} & {\tiny EntityIdAttribute} & {\tiny ENTITY_ID_ATTRIBUTE} & {\tiny -9} & {\tiny -9 \\ \hline
{\tiny C} & {\tiny char*} & {\tiny EntityNameAttribute} & {\tiny ENTITY_NAME_ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny pIntIntervalTab*} & {\tiny TimeDomainAttribute} & {\tiny TIME_DOMAIN_ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny pIntTab* } & {\tiny AnonymousIntTabAttribute} & {\tiny ANONYMOUS_INT_TAB__ATTRIBUTE} & & \\ \hline
{\tiny C} & {\tiny pFloatTab*} & {\tiny AnonymousFloatTabAttribute} & {\tiny ANONYMOUS_FLOAT_TAB_ATTRIBUTE} & & \\ \hline
{\tiny V} & {\tiny int} & {\tiny AnonymousIntAttribute} & {\tiny ANONYMOUS_INT_ATTRIBUTE} & & \\ \hline
{\tiny V} & {\tiny float} & {\tiny AnonymousFloatAttribute} & {\tiny ANONYMOUS_FLOAT_ATTRIBUTE} & & \\ \hline
{\tiny V} & {\tiny double} & {\tiny AnonymousDoubleAttribute} & {\tiny ANONYMOUS_DOUBLE_ATTRIBUTE} & & \\ \hline
{\tiny V} & {\tiny char*} & {\tiny AnonymousStringAttribute} & {\tiny ANONYMOUS_STRING_ATTRIBUTE} & & \\ \hline
{\tiny V} & {\tiny Entity*} & {\tiny AnonymousEntityAttribute} & {\tiny ANONYMOUS_ENTITY_ATTRIBUTE} & & \\ \hline
\end{tabular}
\\

  Note : ne pas confondre le nom C++ de la classe et le "nom de classe" (de la classe). Par exemple, le "nom de classe" de la classe C++ [C]TypeAttribute[/C] est "typeAttribute". C'est la valeur retournée par le service [Y]ClassName()[/Y] et valué par le service [Y]ClassName(char*)[/Y]. De façon générale, le nom de classe est le nom C++ dans lequel on a passé la première lettre en minuscule.\\
Le symbole de classe est retournée par le service [Y]ClassSymbol()[/Y] et valué par le service [Y]ClassSymbol(UserDescriptorId)[/Y].\\

La sémantique proposée pour ces descripteurs, et l'utilisation possible des descripteurs "anonymes" (e.g. [C]AnonymousIntAttribute[/C]), sont documentées à la page "Définition de types, variable globales et constantes symboliques".\\
*/
//@{


//=X===============X=============================X============================
// Général
//............................................................................

class TypeAttribute : public IntConstantDescriptor
{
public:
  TypeAttribute();
};

class InstantAttribute : public IntConstantDescriptor
{
public:
  InstantAttribute();
};

class DescriptorIdAttribute : public IntConstantDescriptor
{
public:
  DescriptorIdAttribute();
};

class DescriptorNameAttribute : public StringConstantDescriptor
{
public:
  DescriptorNameAttribute();
};

class EntityIdAttribute : public IntConstantDescriptor
{
public:
  EntityIdAttribute();
};

class EntityNameAttribute : public StringConstantDescriptor
{
public:
  EntityNameAttribute();
};

class TimeDomainAttribute : public IntDomainConstantDescriptor
{
public:
  TimeDomainAttribute();
};

class AnonymousIntTabAttribute : public IntTabConstantDescriptor
{
public:
  AnonymousIntTabAttribute();
};

class AnonymousFloatTabAttribute : public FloatTabConstantDescriptor
{
public:
  AnonymousFloatTabAttribute();
};

//=X===============X=============================X============================
// Entity
//............................................................................

/*
class DescIdForSorting : public IntConstantDescriptor
{
public:
  DescIdForSorting();
};

class IntDescriptorForSorting : public IntConstantDescriptor
{
public:
  IntDescriptorForSorting();
};

class FloatDescriptorForSorting : public FloatConstantDescriptor
{
public:
  FloatDescriptorForSorting();
};
*/

//=X===============X=============================X============================


#endif  // _CORECONSTDESCRIPTOR_HH
