//============================================================================
// FILE: DesignOfExperiment.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des des specifications
//              de plans d'experience
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _EXPERIMENTALDESIGN_HH
#define _EXPERIMENTALDESIGN_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name CDPFORTEXLes plans d'expérience
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include DesignOfExperiment.h
[/P]
\end{itemize}\label{defDesignOfExperiment}

{\large {\tt Simulation isolée ...}} \\

Une commande telle que ... :
\begin{verbatim}[P]./main  sim1.par  sim1.sim  sim1.osp  sim1.str  sim1.dir[/P]
\end{verbatim}
... lance une simulation isolée, c'est-à-dire que le comportement du système étudié est simulé une seule fois entre le début et la fin de la période spécifiée (normalement dans [F]sim1.sim[/F]). L'état initial su système et les éléments du contexte sont précisés normalement dans les fichiers [F]sim1.par[/F] et [F]sim1.str[/F], pour la partie externalisée.\\

Des options de commande sont disponibles. Certaines sont prédéfinies dans \BD, d'autres peuvent l'être dans les couches applicatives. La liste en est fournie par l'option '-h' : 
\begin{verbatim}
[P]./main -h[/P]
\end{verbatim}

{\large {\tt ... et plan d'expérience :}}\\

Un élément complémentaire de la ligne commande ('-EXPE', voir la page 'Mise en oeuvre ...') permet d'enchaîner automatiquement un ensemble de simulations isolées, avec les motifs suivants :
\begin{itemize}
\item la répétition, N fois, de la simulation isolée, avec une situation initiale et un contexte spécifiés de manière identique pour chacune des N simulations. Cette action n'a d'intérêt que si au moins un événement est aléatoire ou au moins un processus est stochastique ;
\item l'enchaînement de M simulations dont les situations initiales et les contextes sont individuellement explicités ;
\item l'enchaînement de P simulations dont les situations initiales et les contextes sont construits par croisement de modalités de situation initiale et/ou de contexte ;
\item la répétition, N fois, de l'enchaînement des M ou des P simulations définies ci-dessus.
\end{itemize}

Il est important de noter d'ores et déjà que la situation initiale et le contexte seront caractérisés, pour la réalisation de ces enchaînements, par les valeurs d'un jeu de paramètres (classe [C]Parameter[/C]), dont on sait qu'ils sont déclarés et valués dans le fichier [F]*.par[/F] désigné dans la ligne de commande. Par des références à ces paramètres dans les fichiers [F]*.sim[/F] et [F]*.str[/F], on peut définir la situation initiale ou le contexte de simulation par, respectivement, des "paramètres de simulation" et des éléments de la structure du système.\\

A titre d'exemple, soit une base de connaissance comportant les paramètres suivants, entre autres : ("annee" "civile"), ("lieu" "etude"), ("valeur" "X"). Pour une simulation isolée, le fichier des "paramètres du système" ([F]*.par[/F]) contient les lignes suivantes :
\begin{verbatim}
[P]"annee" "civile" <- 2006;
"lieu" "etude" <- "paris"
"valeur" "X" <- 12.34;
[/P]\end{verbatim}

Un enchaînement de M=3 simulations explicitées peut être ainsi défini :
\begin{verbatim}
[P]{(2006 "Paris" 12.34) (2007 "Montpellier" 23.45) (2008 "Lille" 34.56)}
[/P]\end{verbatim}

Un enchaînement de P=6 simulations par croisement de modalités peut être ainsi défini :
\begin{verbatim}
[P]{"annee" "civile" : 2006 2009} X {"lieu" "etude" : "Paris" "Montpellier" "Lille"}
[/P]\end{verbatim}
Dans cette définition, les 6 simulations prennent 12.34 comme valeur du paramètre d'identifiants "valeur" "X".\\

Noter qu'une seule spécification 'FED' doit être écrite.\\
\\

{\large {\tt Comment déclarer un plan d'expérience ?}}\\

De manière informelle, le plan d'expérience ci-dessus, en M simulations explicitées, est exprimé par les lignes suivantes :
\begin{verbatim}
[P]SED sed1 (INT "annee" "civile" : 2006 2007 2008)
            +
            (STRING "lieu" "etude" : "Saint.Germain" Montpellier Lille)
            +
            (REAL "valeur" "X" : 12.34 23.45 34.56);
FED : sed1 (1);
[/P]\end{verbatim}

 Pour mémoire, SED est un acronyme de 'SerialDesignOfExperiment', et FED un acronyme de 'FactorialDesignOfExperiment'. Le 'FED' est ici un plan factoriel dégénéré, puisqu'il ne croise le 'SED' sed1 avec aucun autre. Sa spécification est cependant nécessaire. On ne souhaite effectuer qu'une seule simulation isolée pour chacune des trois modalités. On note que les trois listes correspondant aux trois paramètres comportent, nécessairement, le même nombre de valeurs.\\

Quant au plan d'expérience en P simulations par croisement, il est exprimé par les lignes suivantes :
\begin{verbatim}
[P]SED a_c (INT "annee" "civile" : 2006 2009);
SED l_e (STRING "lieu" "etude" : "Saint.Germain" Montpellier Lille);

FED : a_c * l_e (100);
[/P]\end{verbatim}

Le 'FED' prévoit d'effectuer 100 répétitions de chacune des 6 simulations obtenues par croisement des 2 modalités sur "annee" "civile" et des 3 modalités sur "lieu" "etude".\\

Si on avait souhaité que chaque lieu d'étude soit associé à une valeur particulière de X, on aurait écrit :
\begin{verbatim}
[P]SED a_c (INT "annee" "civile" : 2006 2009);
SED l_e (STRING "lieu" "etude" : "Saint.Germain" Montpellier Lille)
        +
        (REAL "valeur" "X" : 12.34 23.45 34.56);

FED : a_c * l_e (100);
[/P]\end{verbatim}
\\

{\large {\tt Spécification formelle un plan d'expérience :}}\\

Le lexique et la syntaxe de l'expression d'un plan sont les suivants :
\begin{verbatim}
[P]<fichier>               ::= <spec>*
<spec>                  ::= ligne_trace | ligne_com | <spec_sed> | <spec_fed>
 
<ligne_com>             ::= <symbole_com> <any>
<symbole_com>           ::= //
<any>                   ::= .* 
<ligne_trace>           ::= TRACE <mode_trace> <fin>
<mode_trace>            ::= PARSING | NO_PARSING
<fin>                   ::= ;

<spec_sed>              ::= SED <nom_sed> : <jeux_valeurs> <fin>
<nom_sed>               ::= <chaine_simple>
<jeux_valeurs>          ::= <jeu_valeurs> | <jeux_valeurs> + <jeu_valeurs> 
<jeu_valeurs>           ::= (<type> <ident> <ident> : <liste_valeurs>)
<type>                  ::= INT | REAL | STRING
<ident>                 ::= <chaine_simple> | <chaine_minmaj_spec>
<liste_valeurs>         ::= <valeur> | <liste_valeurs> <val>
<val>                   ::= <entier> | <reel> | <str_val>
<str_val>               ::= <chaine_simple> | <chaine_minmaj_spec>

<spec_fed>              ::= <spec_fed_1> | <spec_fed_2>
<spec_fed_1>            ::= FED : <liste_seds> <fin>
<spec_fed_2>            ::= FED : <liste_seds> (<entier>) <fin>
<liste_seds>            ::= <nom_sed> | <liste_seds> * <nom_sed>

<chaine_simple>         ::= <chaine_maj> | <chaine_minmaj>
<chaine_maj>            ::= [A-Z][A-Z0-9\_]*
<chaine_minmaj>         ::= [a-zA-Z][a-zA-Z0-9\_]*
<chaine_minmaj_spec>    ::= "\""[a-zA-Z][a-zA-Z0-9\_\.]*"\""
<entier>                ::= -?[0-9]+
<reel>                  ::= <reel_f> | <reel_g>
<reel_f>                ::= -?[0-9]*[\.\,][0-9]*
<reel_g>                ::= -?[0-9]+[\.\,][0-9]*E[+-][0-9]+[/P]
\end{verbatim}


*/
//@{

/**@name Mise en oeuvre d'un plan en croisements de modalités

Pour réaliser un plan de simulations, il faut :
\begin{itemize}
\item Ecrire un fichier de description du plan, en suivant les directives de la page 'Les plans d'expérience' ;
\item Lire et interpréter ce fichier en préambule de la série d'exécution des simulations. A cette fin, on ajoutera dans la ligne de commande une option -EXPE, de la manière suivante :
\begin{verbatim}
[P]./main ..... -EXPE <fichier>[/P]
\end{verbatim}
où <fichier> est le fichier de description du plan d'expérience.
\end{itemize}

La présence de cette option (comme c'est aussi le cas pour les autres composants de la ligne de commande) doit être d'une part reconnue et d'autre part exploitée par le programme principal de l'application, la fonction [Y]main(...)[/Y]. \\

La reconnaissance est faite automatiquement par la partie générale, [Y]ParseMainArguments()[/Y], définie dans BD, de la fonction [Y]App_ParseMainArguments()[/Y]. Cette fonction est le premier service invoqué dans la fonction [Y]main(...)[/Y]. La rencontre de l'option '-EXPE' provoque la création d'une instance de la classe [C]FactorialDesignOfExperiment[/C]. Cette classe restera unique, puisqu'une seule spécification 'FED' doit être écrite dans le fichier.\\

L'enchaînement des simulations est provoqué, dans la fonction [Y]main(...)[/Y], par l'adressage itératif du message prédéfini [Y]NextSimulationSettings[/Y] à l'instance de [C]FactorialDesignOfExperiment[/C]. Ce service établit la situation initiale et le contexte de la prochaine simulation. L'itération s'arrête quand [Y]NextSimulationSettings[/Y] s'aperçoit que toutes les modalités du plan d'expérience ont été traitées. \\

L'instance de [C]FactorialDesignOfExperiment[/C] est renvoyée par l'appel à la fonction globale prédéfinie [Y]GetDesignOfExperiment[/Y].\\

Le codage proposé ci-dessous, et généré par défaut par l'interface Solfege, présente une boucle d'itération compatible avec l'absence de l'option '-EXPE' dans la ligne de commande. Puisqu'on est alors dans le cas d'une simulation isolée, on donne la valeur {\em 1} au nombre de répétitions à effectuer et on stoppe immédiatement l'itération en donnant la valeur {\em vrai} à sa variable de contrôle ([Y]isExpeOver[/Y]).\\

Avant de débuter l'itération, on lit une (et donc une seule) fois le fichier des paramètres du système (par [Y]ParseParameterFile[/Y]). C'est ensuite [Y]NextSimulationSettings[/Y] qui surchargera la valeur des paramètres mis en jeu dans le plan d'expérience, les autres restant constants. Les structures allouées à cette étape (les instances de la classe [C]Parameter[/C]) sont désallouées au sortir de la boucle d'itération ([Y]DeleteParameters[/Y]). On fait de même pour les structures dédiées à la mise en oeuvre dun plan d'expérience ([Y]DeleteExpeSpecifications[/Y]).\\

Le corps de l'itération consiste alors seulement à lire et exploiter les quatre autres fichiers déclaratifs. Pour assurer la rémanence des paramètres, il ne faut pas inscrire dans le fichier des directives la désallocation des paramètres.\\

\begin{verbatim}
[P]int main (int argc, char* argv[]) {
	App_ParseMainArguments(argc, argv);
	ParseParameterFile(argv[1]);

	int nbRepl = 1;
	FactorialDesignOfExperiment* pFEDS = GetDesignOfExperiment();
	if(pFEDS) {
		//pFEDS->Display();
	}
	Boolean isExpeOver = FALSE;
	while(! isExpeOver) {
		nbRepl = 1;  // default value if no FEDS
		if(pFEDS)
			nbRepl = pFEDS->NextSimulationSettings();

		if(! pFEDS || (nbRepl < 0))  //no FEDS or FEDS is over
			isExpeOver = TRUE;

		for(int i=0;i<nbRepl;i++) {
			InitSimulation();
			ParseSimulationFile(argv[2]);
			ParseOutputSpecFile(argv[3]);
			ParseStructureFile(argv[4]);
			ParseDirectiveFile(argv[5]);
		}
	}
	DeleteParameters();
	DeleteExpeSpecifications();
	return 0;
}[/P]
\end{verbatim}



*/
//@{
          /// La classe des plans factoriels
class FactorialDesignOfExperiment   // _no swig_
{
public:

  /* *@name LIFECYCLE */
  // @{
          // Constructeur de base. 
  FactorialDesignOfExperiment();  // _no swig_
          // Destructeur invoqué dans la fonction prédéfinie de destruction des paramètres.
   ~FactorialDesignOfExperiment();   // _no swig_
// @}          

  /**@name  OPERATIONS */
  //@{
   /// Teste s'il existe encore une modalité non traitée du plan d'expérience. Si oui, surcharge les paramètres en jeu avec les valeurs caractérisant cette modalité, et renvoie le nombre de répétitions de la simulation ainsi initialisée. Sinon, renvoie -9.
   int NextSimulationSettings();
   //
   //void Initialize(); // obsolète
   /// Affiche sur la sortie standard la structure du plan d'expérience.
   void Display();
  //@}         

  /* *@name  ACCESS */
  // @{
   //
   //int        GetNbFactors();  // unused
   //
   void       SetNbFactors(int val);
   //
   int        GetNbReplications();
   //
   void       SetNbReplications(int val);
   //
   //int        GetCurrentSerialExpeDesignIndex();  // unused
   //
   void       SetCurrentSerialExpeDesignIndex(int val);
   //
   void       AddElement(SerialDesignOfExperiment*);
   //
   //SerialDesignOfExperiment*       GetElement(int k);  // unused
   //
   //Tableau<SerialDesignOfExperiment*> GetFactorTab();  // unused
  // @}

protected:
   
   int               mNbFactors;
   int               mNbReplications;
   int               mCurrentSerialExpeDesignIndex;

   Tableau<SerialDesignOfExperiment*> mpSerialExpeDesignTab;

private:
};  // _end class_

//-X///////////////X/////////////////////////////X//////////////////////////// 
// FONCTIONS  
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Fonctions\\

On recommande de désallouer les structures dédiées à la mise en oeuvre d'un plan d'expérience, une fois celui-ci complètement traité.\\

Il s'agit au premier chef de l'unique instance de la classe [C]FactorialDesignOfExperiment[/C].\\

Il se trouve que cette instance encapsule d'autres structures, celles liées aux éléments du plan factoriel et créées à l'occasion de la lecture des clauses 'SED ...' dans le fichier de description du plan. Ces structures sont intégralement gérées, et leurs services exclusivement exploités par le moteur de simulation. Elle n'apparaissent donc pas dans cette documentation 'publique' de /BD. Elles sont désallouées par le service qui désalloue l'instance de la classe [C]FactorialDesignOfExperiment[/C]. 
*/
//@{

        //
void DeleteSerialExpeSpecifications();
        /// Désalloue les structures créées pour la déclaration puis la gestion du plan d'expérience. 
void DeleteExpeSpecifications();
//@}
  
//@}  // fin plans croises

/* *@name Les plans en série de jeux de modalités
\\
*/
// @{

          // / La classe des plans sériels
class SerialDesignOfExperiment   // _no swig_
{
public:

  /* *@name LIFECYCLE */
  // @{
          // Constructeur de base. 
  SerialDesignOfExperiment();  // _no swig_
          // Destructeur invoqué dans la fonction prédéfinie de destruction des paramètres.
   ~SerialDesignOfExperiment();   // _no swig_
// @}               

  /* *@name  OPERATIONS */
  // @{
   ///
   Boolean NextSimulationSettings();
   //
   //void Initialize(); // obsolète
   ///
   void Display();
   ///
   void DisplayIds();
  // @}       

  /* *@name  ACCESS */
  // @{
   ///
   char*      GetName();
   ///
   void       SetName(char* val);
   ///
   //int        GetNbVariables();  // unused
   ///
   void       SetNbVariables(int val);
   ///
   //int        GetNbModalities();  // unused
   ///
   void       SetNbModalities(int val);
   ///
   int        GetCurrentModalityIndex();
   ///
   void       SetCurrentModalityIndex(int val);
   ///
   void       IncrCurrentModalityIndex();
   ///
   TableauString*   GetParamIds1();
   ///
   TableauString*   GetParamIds2();
   ///
   pIntTab*   GetParamTypes();
   ///
   //UnifiedValue* GetUnifiedValue(int, int);  // unused
   ///
   void AddRow();
   ///
   void AddUnifiedValue(int, UnifiedValue*);
  // @}

    // comptage instances
    static void                  AddInstance(SerialDesignOfExperiment*);
    static Tableau<SerialDesignOfExperiment*>         mpInstanceTab;
    static int                                        mInstanceNumber;

protected:

   char*             mName;
   TableauString*    mParamIds1;
   TableauString*    mParamIds2;
   pIntTab*          mParamTypes;

   int               mNbVariables;
   int               mNbModalities;
   int               mCurrentModalityIndex;

   Tableau<Tableau<UnifiedValue*>*>*    mValueDTab;

private:
};  // _end class_

// @} // fin plans sériels

//@} // fin plans d'experiencs


#endif  // _EXPERIMENTALDESIGN_HH
