//////////////////////////////////////////////////////////////////////////////
// FILE: MetaClass.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des classes d'entités
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
///X///////////////X/////////////////////////////X////////////////////////////

#ifndef _METACLASS_HH
#define _METACLASS_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
// 
#include "CoreTypes.h"

class MetaClass 
{
public:
// EXCEPTIONS


// LIFECYCLE

   MetaClass();                                  // constructor
   MetaClass(UserClassId, char*);                // constructor
   ~MetaClass();                                 // destructor

// OPERATORS

   const
   MetaClass&       operator=(const MetaClass&);  // assignment operator

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const MetaClass&) const {return TRUE;};
  Boolean         operator > (const MetaClass&) const {return TRUE;};

// OPERATIONS 

   void             AddInstance(BasicEntity*);
   void             RemoveInstance(BasicEntity*);

// ACCESS

   UserClassId      ClassId() const; 
   char*            ClassName() const; 
   int              InstanceNumber() const;
   int              InstanceCreation() const; 
   int              InstanceDeletion() const; 
      // attention! on retourne l'adresse de l'attribut (méthode non const)
   pEntityTab*      InstanceTab(); 

   BasicEntity*     GetInstance(int);

   void             ClassId(UserClassId);
   void             ClassName(char*);
   void             InstanceNumber(int);

   void             SetParentClassId(UserClassId);
   void             SetParentClassIds(UserClassId, UserClassId);
   void             AddParentClassId(UserClassId);
   void             AddChildClassId(UserClassId);


   UserClassIdTab   ParentIds() const;
   UserClassIdTab   ChildIds() const;
   UserClassIdTab   AncestorIds() const;
   void             AncestorIds(UserClassIdTab& result) const; 
   void             TerminalDescendantIds(UserClassIdTab& result) const;
   UserClassIdTab   DescendantIds() const;
   void             DescendantIds(UserClassIdTab& result) const;

   Boolean          IsParentOf(UserClassId) const;
   Boolean          IsChildOf(UserClassId) const;
   Boolean          IsAncestorOf(UserClassId) const;
   Boolean          IsDescendantOf(UserClassId) const;

   // Cette fonction établit les liens hiérarchiques réciproques entre la classe de la présente instance et ses classes mères. Il faut avoir au prélable déclaré les classes parentes avec {\bf SetParentclassId} ou {\bf SetParentclassIds}.
   void             SetOneLevelHierarchy();

   // Cette fonction établit les liens hiérarchiques réciproques entre la classe de la présente instance et ses classes ascendantes. Il faut avoir au prélable déclaré les classes parentes avec {\bf SetParentclassId} ou {\bf SetParentclassIds}.
   void             SetTotalHierarchy();
    
      // Si la classe 'this' hérite (à quelque profonder que se soit) de plus d'une classe (héritage multiple), retourne la liste des symboles des classes en position de parent multiple de 'this' (les plus haut placés dans la hiérarchie). Sinon, retourne une liste dont le seul élément est le symbole de ca classe dethis.
   UserClassIdTab*   GetUpperMultipleParents();
   
   // Spécifie si la classe est instantiable (par défaut) ou non par un développeur.
   void              UserInstantiable(Boolean arg);

   // Renvoie vrai ssi la classe est instantiable par un développeur.
   Boolean           IsUserInstantiable() const;

   // Spécifie si lon construit la liste des instances de la métaclasse.
   void              ExistsInstanceTab(Boolean arg);

   // Renvoie vrai ssi on construit la liste des instances.
   Boolean           ExistsInstanceTab() const;

// INQUIRY

protected:

   UserClassId       mClassId;
   char*             mClassName;

   UserClassIdTab    mParentClassIdTab;
   UserClassIdTab    mAncestorClassIdTab;
   UserClassIdTab    mChildClassIdTab;
   UserClassIdTab    mDescendantClassIdTab;

   bool              mTotalHierarchyInstalled;

   int               mInstanceNumber;
   int               mInstanceCreation;
   int               mInstanceDeletion;
   pEntityTab        mpInstanceTab;

   Boolean           mUserInstantiable;  // + JPR 070730
   Boolean           mExistsInstanceTab;  // + JPR 070730

private:

   friend class BasicEntity;
   friend class DescriptedEntity;
   friend class Entity;
};

// INLINE METHODS


// EXTERNAL REFERENCES

#endif  // _METACLASS_HH
