//============================================================================
// FILE: C_CoreArgument.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des diverses classes d'arguments de Control-Diese
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_COREARGUMENT_HH
#define _C_COREARGUMENT_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Argument.h"

/**@name Les arguments prédéfinis des méthodes prédéfinies
\begin{tabular}{lllll}
type&nom C++&symbole de classe&nom de classe&sens de la valeur\\
&OpenSonRank&OPEN_SON_RANK&openSonRank& \\
&ClosedSonRank&CLOSED_SON_RANK&ClosedSonRank& \\
&OperationTarget&OPERATION_TARGET&opTarget& \\
&TransferOperationSource&TRANSFER_OPERATION_SOURCE&transferOperationSource& \\
&TransferOperationDestination&TRANSFER_OPERATION_DESTINATION&transferOperationDestination& \\
&SingleActivitySetArgument&SINGLE_ACTIVITY_SET_ARGUMENT&singleActivitySetArgument& \\
&SingleListArgument&SINGLE_LIST_ARGUMENT&singleListArgument& \\
&TimeDomainArgument&TIME_DOMAIN_ARGUMENT&timeDomainArgument& \\
&ActivitySetsArgument&ACTIVITY_SETS_ARGUMENT&activitySetsArgument& \\
&ActivitySetArgument&ACTIVITY_SET_ARGUMENT&activitySetArgument& \\
\end{tabular}
 */
/* obsolète
&QuantityId&QUANTITY_ID&quantityId& \\
&QuantityOption&QUANTITY_OPTION&quantityOption& \\
&QuantityIntegerValue&QUANTITY_INTEGER_VALUE&quantityIntegerValue& \\
&QuantityFloatValue&QUANTITY_FLOAT_VALUE&quantityFloatValue& \\

&ActivityArgument&ACTIVITY_ARGUMENT&activityArgument& \\
&ActivityListArgument&ACTIVITY_LIST_ARGUMENT&activityListArgument& \\
&ResourceListArgument&RESOURCE_LIST_ARGUMENT&resourceListArgument& \\
*/
//@{

//=X===============X=============================X============================
// pour les Méthodes d'Activity/Activities/Operation
//............................................................................

class OpenSonRank : public IntArgument
{
public:
   OpenSonRank();
};

class ClosedSonRank : public IntArgument
{
public:
   ClosedSonRank();
};

//=X===============X=============================X============================
// pour les Méthodes d'Operation
//............................................................................

class OperationTarget : public EntityArgument
{
public:
   OperationTarget();
};

class TransferOperationSource : public EntityArgument
{
public:
   TransferOperationSource();
};

class TransferOperationDestination : public EntityArgument
{
public:
   TransferOperationDestination();
};

//=X===============X=============================X============================
// pour les Méthodes de ressources
//............................................................................

/*
class InOutAccessMode : public IntArgument  // obsolète
{
public:
   InOutAccessMode();
};

class InOutStockIntegerQuantity : public IntArgument  // obsolète
{
public:
   InOutStockIntegerQuantity();
};

class InOutStockRealQuantity : public DoubleArgument  // obsolète
{
public:
   InOutStockRealQuantity();
};

class QuantityId : public IntArgument  // obsolète
{
public:
   QuantityId();
};

class QuantityOption : public IntArgument  // obsolète
{
public:
   QuantityOption();
};

class QuantityIntegerValue : public IntArgument  // obsolète
{
public:
   QuantityIntegerValue();
};

class QuantityFloatValue : public FloatArgument  // obsolète
{
public:
   QuantityFloatValue();
};
*/

//=X===============X=============================X============================


//=X===============X=============================X============================
// Goal
//............................................................................

class SingleActivitySetArgument : public EntityArgument
{
public:
   SingleActivitySetArgument();
};

class SingleListArgument : public EntityTabArgument
{
public:
   SingleListArgument();
};
/*
class ActivityArgument : public EntityArgument  // obsolète
{
public:
   ActivityArgument();
};

class ActivityListArgument : public EntityTabArgument  // obsolète
{
public:
   ActivityListArgument();
};

class ResourceListArgument : public EntityTabArgument  // obsolète
{
public:
   ResourceListArgument();
};
*/


//=X===============X=============================X============================
// Target_Goal + Anti_Target_Goal
//............................................................................

class TimeDomainArgument: public IntDomainArgument
{
public:
   TimeDomainArgument();
};

//=X===============X=============================X============================
// Mono_Goal_Utilite_Criterion
//............................................................................

class ActivitiesResourcesBlockArgument: public EntityArgument
{
public:
   ActivitiesResourcesBlockArgument();
};

class ActivitySetsArgument: public EntityArgument
{
public:
   ActivitySetsArgument();
};

//=X===============X=============================X============================
// Manager (provisoirement, avant attachement aux ARB)
//............................................................................

class ActivitySetArgument: public EntityArgument
{
public:
   ActivitySetArgument();
};




//@}  // fin arg Prédef

//@}  // fin méth + arg Prédef  (ouverture dans C_CoreMethod.h)


#endif  // _C_COREARGUMENT_HH
