//============================================================================
// FILE: C_OperatedObjectSpecification.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des spécifications d'objet opéré (par une activité)
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_OPERATEDOBJECTSPECIFICATION_HH
#define _C_OPERATEDOBJECTSPECIFICATION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"
#include "C_AggregatedResource.h"
#include "C_SingleResource.h"
#include "C_TransferInformation.h"

// EXTERNAL REFERENCES

/**@name Spécification d'une activité */
//@{    // fermeture dans PerformerSpecification

/**@name Spécification des objets opérés

\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_OperatedObjectSpecification.h} \end{itemize}
*/ 
//@{
/**@name Fonctions d'aggrégation des objets opérés en une ressource unique*/
//@{
          /// 
HomogeneousAggregatedCapacitatedResource* MakeAggregatedCapacitatedResource(AllocatedOperatedObjectSet* pSet);
          /// 
HomogeneousAggregatedCapacitatedResource* MakeAggregatedCapacitatedResource(pEntityTab* pSet);
//@}

//@{
          /// 
class OperatedObjectSpecification : public Entity
{
 public:

          // Constructeur de base.
   OperatedObjectSpecification();

           // 
   //void       Develop(PrimitiveActivity* pAct);
  void       Develop(PrimitiveActivity* pAct, BasicEntity* pSourceOOSet);
           // 
  pEntityTab* MakeDisjunctionByANYONE(pEntityTab*, int);
  pEntityTab* MakeSourceDisjunctionByANYONE(pEntityTab*, int, bool, 
					    TransferInformation*);
  pEntityTab* MakeDestDisjunctionByANYONE(PrimitiveActivity*, 
					  pEntityTab*, int, bool, 
					  TransferInformation*,
					  BasicEntity*);

           // 
  pEntityTab* MakeDisjunctionByMAX(pEntityTab*);
  pEntityTab* MakeSourceDisjunctionByMAX(pEntityTab*, bool, 
					 TransferInformation*);
  pEntityTab* MakeDestDisjunctionByMAX(PrimitiveActivity*,
				       pEntityTab*, bool, 
				       TransferInformation*,
				       BasicEntity*);
  
           // 
  pEntityTab* MakeDisjunctionByALL(pEntityTab*);
  pEntityTab* MakeSourceDisjunctionByALL(pEntityTab*, bool, 
					 TransferInformation*);
  pEntityTab* MakeDestDisjunctionByALL(PrimitiveActivity*, 
				       pEntityTab*, bool, 
				       TransferInformation*,
				       BasicEntity*);

           // 
  pEntityTab* MakeDisjunctionByDISJ(pEntityTab*);
  pEntityTab* MakeSourceDisjunctionByDISJ(pEntityTab*, bool, 
					  TransferInformation*);
  pEntityTab* MakeDestDisjunctionByDISJ(PrimitiveActivity*, 
					pEntityTab*, bool, 
					TransferInformation*,
					BasicEntity*);
  
           // 
  pEntityTab* MakeDisjunctionBySETS(pEntityTab*, int);
  pEntityTab* MakeSourceDisjunctionBySETS(pEntityTab*, int, bool, 
					  TransferInformation*);
  pEntityTab* MakeDestDisjunctionBySETS(PrimitiveActivity*, 
					pEntityTab*, int, bool, 
					TransferInformation*,
					BasicEntity*);


          // Renvoie la liste (dite "liste opérée") des {\bf OperatedObject} résultant de l'invocation de {\bf MakeInstructionList} et sur lesquels l'instruction est mise en oeuvre.
pEntityTab* GetOperatedObjectList();

          // Cette méthode est invoquée par {\bf ActInstructionList}. Elle renvoie l'{\bf OperateObject} sur lequel l'opération passée en argument est en cours de mise en oeuvre ou, à défaut, le prochain sur lequel elle doit être mise en oeuvre. C'est le premier, dans l'ordre de construction de la liste opérée, à ne pas l'opération dans la valeur du descripteur FULLY\_OPERATED\_BY. 
OperatedObject*    CurrentOperatedObject(Operation*);

          /// 
//OperatedObject*     CurrentTransferDestination(TransferInformation*);
 OperatedObject*     CurrentTransferDestination(UserDescriptorId);

           // Renvoie TRUE si la spécification requiert une liste développée non vide, c'est-à-dire si la fonction est de type ANYONE ou SETS avec un effectif non nul.
   Boolean    ImpliesNonVoidDeveloppedList();

 private:
 };
//@}   // fin classe OperatedObjectSpecification


//@}  // fin fonctions + classe

#endif  // _C_OPERATEDOBJECTSPECIFICATION_HH
