//============================================================================
// FILE: C_OperationSpecification.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des spécifications d'opération (dans une activité)
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_OPERATIONSPECIFICATION_HH
#define _C_OPERATIONSPECIFICATION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"
#include "C_SingleResource.h"
#include "C_OperationResourceSpecification.h"

// EXTERNAL REFERENCES

/**@name Spécification de l'opération

\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_OperationSpecification.h} \end{itemize}
*/ 
//@{
        ///
class OperationSpecification : public Entity
{
 public:
          // Constructeur de base.
   OperationSpecification();

           // Etablit la valeur du descripteur de symbole SPECIFIED_OPERATION. C'est l'instance d'{\bf Operation} issue de l'expansion de l'{\bf EntitySpec} qui est la valeur du descripteur de symbole OPERATION_ENTITY_SPEC_ATTRIBUTE.\index{méthodes!DevelopOperation}  
           /// Etablit la valeur du dscripteur de symbole SPECIFIED_OPERATION. C'est l'instance d'{\bf Operation} issue de l'instantiation de la sous-classe d'{\bf Operation} dont le symbole est la valeur du descripteur de symbole OPERATION_CLASS_ID.\index{méthodes!DevelopOperation}  
   void     DevelopOperation();

           /// Génère une disjonction de listes de ressources propres d'opération, sous la forme d'une liste d'AllocatedOperationResourceSet. On la construit en combinant les disjonctions de listes de ressources propres d'opération issues du développement de chaque élément des ressources requises (descripteur REQUIRED\_RESOURCE) de l'opération. L'allocation pour l'opérateur doit avoir été générée et passée en paramètre.\index{méthodes!DevelopResourceSpecList}   
   void     DevelopResourceSpecList(pEntityTab*);

           /// Génère une liste de ressources propres d'opération, sous la forme d'une liste d'AllocatedOperationResourceSet, pour un élément des ressources requises (descripteur REQUIRED\_RESOURCE) de l'opération. L'allocation pour l'opérateur doit avoir été générée et passée en paramètre.\index{méthodes!DevelopResourceSpec}    
pEntityTab* DevelopResourceSpec(OperationResourceSpecification* rscSpec_k,
								pEntityTab* perfAlloc);

           /// Renvoie le symbole de classe de la classe d'opération recherchée. Si un symbole n'a pas été précisé, on l'établit à partir du nom de classe. Si celui-ci n'a pas été fourni, l'exception {\bf NoInformationForReferenceClass} est levée.\index{méthodes!GetOperationClassId}  
   UserClassId   GetOperationClassId();
            
           ///    
   Boolean ExistsOperationResourceRequisition(); // gcc 4.1

           // Renvoie TRUE si la spécification requiert une liste développée non vide pour les ressources propres de l'opération en jeu, c'est-à-dire si la fonction est de type ANY ou SETS avec un effectif non nul pour au moins une spécification de ressource propre.
   Boolean    ImpliesNonVoidDeveloppedList();

 private:
 };


//@}  // fin OperationSpecification

#endif  // _C_OPERATIONSPECIFICATION_HH
